/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package logging;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.logging.Level;

/**
 * Der Logger dient dazu, Ausgaben der Engine moeglichst strukturiert und
 * uebersichtlich zu gestalten.
 * 
 * @author nico3000, Erik Wittkorn
 */
public class Logger {
    /**
     * Einzige Instanz des Loggers.
     */
    public static final Logger INSTANCE = new Logger();
    
    /**
     * Ausgabestream zur Logdatei.
     */
    private BufferedWriter logFileWriter = null;

    /**
     * Privater Konstruktor entsprechend dem Singletonprinzip.
     */
    private Logger() {
    }
    
    /**
     * Oeffnet den Stream zur Logdatei.
     * @param filePath Der Pfad zur Datei
     * @return <tt>true</tt>, falls erfolgreich
     */
    public boolean open(String filePath) {
        if(this.logFileWriter != null) {
            try {
                this.logFileWriter.close();
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(Logger.class.getName()).log(Level.SEVERE, null, ex);
                return false;
            }
        }
        try {
            this.logFileWriter = new BufferedWriter(new FileWriter(filePath));
            this.logFileWriter.write("Nicotopia Engine v1.0 - Logfile\n");
        } catch (IOException ex) {
            java.util.logging.Logger.getLogger(Logger.class.getName()).log(Level.SEVERE, null, ex);
            return false;
        }
        return true;
    }
    
    /**
     * Schliesst den Stream zur Logdatei.
     */
    public void close() {
        if(this.logFileWriter != null) {
            try {
                this.logFileWriter.close();
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(Logger.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
    
    /**
     * Druckt eine Zeile in die Datei und auf die Konsole.
     * @param line Die Zeile, die gedruckt werden soll
     * @param asError Falls <tt>true</tt>, wird die Konsolenausgabe in den
     *                Errorstream geschrieben
     */
    private synchronized void print(String line, boolean asError) {
        (asError ? System.err : System.out).print(line);
        if(this.logFileWriter != null) {
            try {
                this.logFileWriter.write(line);
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(Logger.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            System.err.println("[ERROR] Logger not yet opened. Call open() on newly created Logger instance.");
        }        
    }
    
    /**
     * Druckt eine Information der Form <tt>[INFO] Dateiname, Zeile: text</tt>.
     * Der Offset gibt dabei die Anzahl an Dateien an, die zurueckgegangen
     * werden soll, um die Quelle des Textes an eine andere Stelle zu setzen.
     * @param text Der zu erscheinende Text
     * @param backtrackOffset Der Offset des Backtracking
     */
    public void info(String text) {
        String line = String.format("[INFO] %s%s\n", this.getSrcString(), text);
        this.print(line, false);
    }
    
    /**
     * Druckt eine Warnung der Form <tt>[WARNING] Dateiname, Zeile: text</tt>.
     * Der Offset gibt dabei die Anzahl an Dateien an, die zurueckgegangen
     * werden soll, um die Quelle des Textes an eine andere Stelle zu setzen.
     * @param text Der zu erscheinende Text
     * @param backtrackOffset Der Offset des Backtracking
     */
    public void warning(String text) {
        String line = String.format("[WARNING] %s%s\n", this.getSrcString(), text);
        this.print(line, false);
    }
    
    /**
     * Druckt einen Fehler der Form <tt>[ERROR] Dateiname, Zeile: text</tt>. Der
     * Offset gibt dabei die Anzahl an Dateien an, die zurueckgegangen werden
     * soll, um die Quelle des Textes an eine andere Stelle zu setzen.
     * @param text Der zu erscheinende Text
     * @param backtrackOffset Der Offset des Backtracking
     */
    public void error(String text) {
        String line = String.format("[ERROR] %s%s\n", this.getSrcString(), text);
        this.print(line, true);
    }
    
    /**
     * Liefert den String, der Dateiname und Zeile enthaelt.
     * @param backtrackOffset Der Offset des Backtracking
     * @return Ein String der Form <tt>Dateiname, Zeile:</tt>
     */
    private String getSrcString() {
        StackTraceElement stack[] = Thread.currentThread().getStackTrace();
        String fileLineNumber = "KeineAhnung.java, 42: ";
        
        //TODO: Den StackTrace durchiterieren und den richtigen Dateinamen und die richtige
        //		Zeilenangabe finden und hier zurueckgeben
        //		Lesen von http://docs.oracle.com/javase/1.5.0/docs/api/java/lang/StackTraceElement.html lohnt sich!!
        
        return fileLineNumber;
    }

}
