package logic;

import input.InputListener;
import logging.Logger;
import main.BaseGameLogic;
import main.GameApp;
import model.Level;

import org.lwjgl.util.vector.Vector3f;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import processes.PickupProcess;
import processes.PlayerController;
import util.MageQuestUtil;
import util.MathUtil;
import util.XMLUtil;
import actor.Actor;
import actor.ActorComponent;

import components.PickupComponent;

import event.EventData;
import event.EventListener;
import events.LogicEvents;
import events.MoveEvents;

/**
 * In dieser Klasse befinden sich verschiedene Initialisierungsmethoden und
 * Controller-Methoden, die fuer die Logik des "Spiels" notwendg sind.
 * 
 * @author Erik Wittkorn
 */
public class MageQuestLogic extends BaseGameLogic implements EventListener {
    
	/**
	 * Der EventListener fuer alle Input-Events
	 */
	public static InputListener input = new InputListener();
	/**
	 * Prozess, der fuer die fluessige Bewegung des Spielers zustaendig ist
	 */
    public static PlayerController pController = new PlayerController();
    
    /**
     * Referenz auf den Actor des Spielers
     */
	public static Actor player;
    
	/**
	 * Referenz auf das aktuelle Level
	 */
    public static Level level = null;
	
	@Override
	public boolean init() {
		//  Nicht die Oberklasse vergessen ;)
		boolean success = super.init();
		//  Wegen unseren selbstdefinierten ActorComponents muss ihr Package noch registriert werden
		success = ActorComponent.addComponentPackage("components.");
		//  Der InputHandler wird fuer alle Input-Events registriert
		success = GameApp.getEventManager().register(input,
											MoveEvents.ForwardEvent.ID,
											MoveEvents.BackwardEvent.ID,
											MoveEvents.StrafeLeftEvent.ID,
											MoveEvents.StrafeRightEvent.ID);
		//  Die GameLogic selbst wird fuer Logic-Events registriert
		success = GameApp.getEventManager().register(this,
											LogicEvents.CollectPickupEvent.ID);
		//  Der PlayerController-Prozess wird direkt beim Prozessmanager eingehaengt und dadurch gestartet
		GameApp.getGlobalProcessManager().attachProcess(pController);
		return success;
	}
    
	@Override
	/**
	 * In dieser Methode wird die Level-XML geparst
	 */
	public void loadLevel(String resource) {
        Document doc = GameApp.loadXML(resource);
        // Erste Node ist die <Scene>-Node
        Node node = doc.getFirstChild();
        if(node != null) {
        	// Kinder der Szene koennen Actors sein
            Node actorNode = node.getFirstChild();
            while(actorNode != null) {
                switch (actorNode.getNodeName()) {
                	// Ein Actor kommt an
                	case "Actor" :
                		// Der Actor wird in der Hilfsmethode erstellt
	                    Actor currentActor = this.createActor(XMLUtil.getAttributeAsString(actorNode, "resource"), actorNode);
	                    String name = currentActor.getName();
	                    // Wenn der Name des Actors "player" ist, wird er dem PlayerController
	                    // hinzugefuegt
	                    if (name.equals("player")) {
	                    	pController.setPlayer(currentActor);
	                    	MageQuestLogic.player = currentActor;
	                    }
	                    break;
                	case "Level" :
                		// Eine <Level>-Node wird von der Model-Klasse selbst geparst
                		level = new Level();
                		level.readFromLevelNode(actorNode);
                		break;
                }
                actorNode = actorNode.getNextSibling();
            }
        } else {
            Logger.INSTANCE.warning("Level file does not contain any actors.");
        }
        
        // Nachdem das Level geparst wurde, koennen noch ein paar Einstellungen an den PlayerController weitergegeben werden
        pController.setTileSize(level.getTileSize());
        pController.setPlayerPosition(level.getSpawnPoint().x, level.getSpawnPoint().y, level.getSpawnPoint().z);
        // Nachdem das Level geparst wurde, erstellt eine Hilfsmethode die noetigen Actors
        MageQuestUtil.createActorsFromLevel();
	}
	
	
    /**
     * Erzeugt einen Actor aus einer Node einer xml-Leveldatei.
     * @param node Die Node des Actors
     */
    public static Actor createActor(Node node) {
    	// Die Resource ist die Referenz-XML dieses Actors
        String resource = XMLUtil.getAttributeAsString(node, "resource");
        Actor actor = null;
        if(resource != null) {
            actor = GameApp.getGameLogic().createActor(resource, node);
        }
        return actor;
    }
    
    /**
     * Leitet eine Bewegung an den PlayerController weiter, wenn sie moeglich ist
     * @param direction Richtung der Bewegung
     */
    public static void movePlayer(int direction) {
    	// Hilfsvariablen fuer Informationen des Levels
    	int xOffset = level.getxOffset();
    	int yOffset = level.getyOffset();
    	int zOffset = level.getzOffset();
    	int xOffsetSize = level.getxOffset() + level.getxSize();
    	int yOffsetSize = level.getyOffset() + level.getySize();
    	int zOffsetSize = level.getzOffset() + level.getzSize();
    	int [][][] elements = level.getElements();
    	
    	// Die naechste Position des Spielers wird aufgrund der gewuenschten Bewegungsrichtung berechnet
    	Vector3f nextPosition = pController.calculateNextPlayerPosition(direction);
    	int xPos = (int)nextPosition.x;
    	int yPos = (int)nextPosition.y;
    	int zPos = (int)nextPosition.z;
    	
    	// TODO: Quest 3.3 Kollisionsabfrage einfuegen
		// Die naechste Position wird gesetzt
		pController.movePlayerToPosition(nextPosition.x, nextPosition.y, nextPosition.z);
		// Wenn im Level bei der naechsten Position ein Pickup hinterlegt wurde
		if (elements[MathUtil.clamp(xPos, xOffset, xOffsetSize)][MathUtil.clamp(yPos, yOffset, yOffsetSize)][MathUtil.clamp(zPos, zOffset, zOffsetSize)] == 6) {
			Actor pickupActor = GameApp.getGameLogic().getActor(level.getActorIDs()[MathUtil.clamp(xPos, xOffset, xOffsetSize)][MathUtil.clamp(yPos, yOffset, yOffsetSize)][MathUtil.clamp(zPos, zOffset, zOffsetSize)]);
			// Wird geprueft, ob der Actor an dieser Stelle eine PickupKomponente hat
			if (pickupActor != null && pickupActor.getComponent(PickupComponent.ID) != null) {
				// Wenn ja, wird ein CollectedPickupEvent geworfen
				LogicEvents.CollectPickupEvent pickupEvent = new LogicEvents.CollectPickupEvent(MageQuestLogic.player, pickupActor);
				GameApp.getEventManager().queueEvent(pickupEvent);
			}
		}
    }
    
    /**
     * Leitet eine Rotation an den PlayerController weiter
     * @param direction Richtung der Rotation
     */
    public static void rotatePlayer(int direction) {
    	//TODO: Quest 3.1 3.
    }

	@Override
	/**
	 * Verarbeitet Events, die mit der Logik des Spiels zu tun haben
	 */
	public void trigger(EventData event) {
		switch (event.getId()) {
			case LogicEvents.CollectPickupEvent.ID:
				//  Wenn ein Pickup aufgesammelt wurde, wird der Prozess erstellt und gestartet, der es verbraucht
				LogicEvents.CollectPickupEvent pickupEvent = (LogicEvents.CollectPickupEvent)event;
				PickupProcess pickupProc = new PickupProcess(pickupEvent.getPlayer(), pickupEvent.getPickup());
    			GameApp.getGlobalProcessManager().attachProcess(pickupProc);
    			break;
		}
		
	}

}
