package model;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;

import logging.Logger;
import main.GameApp;

import org.lwjgl.util.vector.Vector3f;
import org.w3c.dom.Node;

import resource.ResHandle;
import resource.Resource;
import util.XMLUtil;

/**
 * Model-Klasse, die Informationen fuer ein MageQuest-Level hat
 * Repraesentation einer .mqll-Datei
 * 
 * @author Erik Wittkorn
 */
public class Level {

	/**
	 * HashMap, die jeder ID, die in der .mqll Datei vorkommt einen Typen zuordnet
	 */
	public static HashMap<Integer, String> idToBlocktype = new HashMap<Integer, String>();
	
	/**
	 * Ausmass des Levels in x-, y- und z-Richtung, in Levelkoordinaten
	 */
	private int ySize = 0;
	private int xSize = 0;
	private int zSize = 0;

	/**
	 * Verschiebt die Actors des Levels und einen beliebigen Wert in Weltkoordinaten
	 */
	private int xOffset = 0;
	private int yOffset = 0;
	private int zOffset = 0;
	
	/**
	 * Groesse eines Blocks in Welt-Koordinaten
	 */
	private int tileSize = 0;
	
	/**
	 * Position, an die der Spieler initial gesetzt wird
	 */
	private Vector3f spawnPoint = new Vector3f();

	/**
	 * Entspricht dem Inhalt der .mqll Datei, beinhaltet die IDs der Elemente in [x][y][z]-Dimensionen
	 */
	int[][][] elements;
	
	/**
	 * Enthaelt die IDs der Actors, die fuer dieses Level erstellt wurden in [x][y][z]-Dimensionen
	 */
	int[][][] actorIDs;
	
	/**
	 * Liest die Informationen des Levels aus der <Level>-Node aus
	 * @param levelNode <Level>-Node
	 */
	public void readFromLevelNode(Node levelNode) {
		
		// Groesseninformationen
		xSize = XMLUtil.getAttributeAsInt(levelNode, "xSize", 0);
		ySize = XMLUtil.getAttributeAsInt(levelNode, "ySize", 0);
		zSize = XMLUtil.getAttributeAsInt(levelNode, "zSize", 0);
		xOffset = XMLUtil.getAttributeAsInt(levelNode, "xOffset", 0);
		yOffset = XMLUtil.getAttributeAsInt(levelNode, "yOffset", 0);
		zOffset = XMLUtil.getAttributeAsInt(levelNode, "zOffset", 0);
		tileSize = XMLUtil.getAttributeAsInt(levelNode, "tileSize", 0);

		// Arrays fuer Element-IDs und die ActorIDs werden initialisiert
		elements = new int[xSize][ySize][zSize];
		actorIDs = new int[xSize][ySize][zSize];
		
		Node layerNode = levelNode.getFirstChild();
		while (layerNode != null) {
			switch (layerNode.getNodeName()) {
				case "Metainformation":
					// In den Metainformationen befinden sich die Zuordnung eines Element-Typs und einer ID
					String file = XMLUtil.getAttributeAsString(layerNode, "file");
					ResHandle handle = GameApp.getResCache().getHandle(new Resource(file));
					if (handle != null) {
						try {
							BufferedReader reader = new BufferedReader(new InputStreamReader(handle.getBufferAsInputStream()));
							String line = null;
							while((line = reader.readLine()) != null) {
								String[] lineElements = line.split("\\s+");
								int id = Integer.valueOf(lineElements[0]);
								String type = lineElements[1];
								idToBlocktype.put(id, type);
							}
						} catch (IOException e) {
							Logger.INSTANCE.warning("Fehler beim Datei laden aufgetaucht: " + e.getMessage());
						}
						handle.unlock();
					}
					break;
				case "SpawnPoint":
					// Der Spawnpunkt wird gesetzt
					int xPos = XMLUtil.getAttributeAsInt(layerNode, "xPos", 0);
					int yPos = XMLUtil.getAttributeAsInt(layerNode, "yPos", 0);
					int zPos = XMLUtil.getAttributeAsInt(layerNode, "zPos", 0);
					this.spawnPoint.set(xPos, yPos, zPos);
					break;
				case "Layer":
					// Jede Ebene des Levels wird bearbeitet
					yPos = XMLUtil.getAttributeAsInt(layerNode, "yPos", 0);
					file = XMLUtil.getAttributeAsString(layerNode, "file");
					handle = GameApp.getResCache().getHandle(new Resource(file));
					if (handle != null) {
						try {
							// Die Datei der Ebene wird geladen
							BufferedReader reader = new BufferedReader(new InputStreamReader(handle.getBufferAsInputStream()));
							int z = 0;
							String line = null;
							while((line = reader.readLine()) != null) {
								// Jede Zeile der Datei wird in ein String-Array gesplittet
								// mit Leerzeichen als Trennsymbol
								String[] lineElements = line.split("\\s+");
								for (int x = 0; x < lineElements.length; x++) {
									// Die String-Zahl wird in ein int geparst und im elements-array an der richtigen Position hinterlegt
									elements[x][yPos][z] = Integer.valueOf(lineElements[x]);
								}
								z++;
							}
						} catch (IOException e) {
							Logger.INSTANCE.warning("Fehler beim Datei laden aufgetaucht: " + e.getMessage());
						}
						handle.unlock();
					}
					break;
			}
			
			
			layerNode = layerNode.getNextSibling();
		}
		
	}

	/**
	 * Setzt die ActorID an einer bestimmten Position im actorIDs-Array
	 * @param x x-Koordinate
	 * @param y x-Koordinate
	 * @param z x-Koordinate
	 * @param actorID zu setzende ID
	 */
	public void setActorID(int x, int y, int z, int actorID) {
		this.actorIDs[x][y][z] = actorID;
	}
	
	/**
	 * Setzt die ActorID an einer bestimmten Position im actorIDs-Array
	 * @param x x-Koordinate
	 * @param y x-Koordinate
	 * @param z x-Koordinate
	 * @param actorID zu setzende ID
	 */
	public void setElementID(int x, int y, int z, int actorID) {
		this.elements[x][y][z] = actorID;
	}
	
	/**
	 * Entfernt einen Levelactor aus dem Elements-Array und actorIDs Array
	 * @param actorID zu entfernende actorID
	 */
	public boolean removeLevelActorWithID(int actorID) {
		boolean found = false;
		for (int xPos = 0; xPos < xSize; xPos++) {
			for (int yPos = 0; yPos < ySize; yPos++) {
				for (int zPos = 0; zPos < zSize; zPos++) {
					if (actorIDs[xPos][yPos][zPos] == actorID) {
						actorIDs[xPos][yPos][zPos] = 0;
						elements[xPos][yPos][zPos] = 0;
						found = true;
					}
				}
			}
		}
		return found;
	}
	
	/**
	 * Getter fuer die einzelnen Level-Variablen
	 */
	public int[][][] getActorIDs() {
		return this.actorIDs;
	}
	
	public int getySize() {
		return ySize;
	}
	
	public int getxSize() {
		return xSize;
	}
	
	public int getzSize() {
		return zSize;
	}
	
	public int getxOffset() {
		return xOffset;
	}
	
	public int getyOffset() {
		return yOffset;
	}
	
	public int getzOffset() {
		return zOffset;
	}
	
	public int getTileSize() {
		return tileSize;
	}
	
	public int[][][] getElements() {
		return elements;
	}
	
	public Vector3f getSpawnPoint() {
		return spawnPoint;
	}
}
