package processes;

import logging.Logger;

import org.lwjgl.util.vector.Vector3f;

import util.MathUtil;
import util.Pose;
import actor.Actor;
import actor.components.TransformComponent;
import process.EngineProcess;

/**
 * Dieser Prozess kuemmert sich um die Bewegung des Spielers.
 * 
 * @author Erik Wittkorn
 */
public class PlayerController extends EngineProcess {
	
	/**
	 * Richtungen fuer die Bewegung
	 */
	public static final int DIR_NO_DIRECTION = 0;
    public static final int DIR_FORWARD = 1;
    public static final int DIR_BACKWARD = 2;
    public static final int DIR_STRAFERIGHT = 3;
    public static final int DIR_STRAFELEFT = 4;
    
    /**
     * Richtungen fuer die Rotation
     */
    public static final int ROTATE_LEFT = 0;
    public static final int ROTATE_RIGHT = 1;
    public static final int ROTATE_UP = 2;
    public static final int ROTATE_DOWN = 3;
	
    /**
     * Referenz auf die Pose des Spielers um seine Position
     * und Rotation manipulieren zu koennen
     */
	private Pose playerPose;
	
	/**
	 * Aktuelle Position des Spielers im PlayerController
	 */
	private Vector3f position;
	
	/**
	 * Aktuelle Rotation des Spielers als Vektor,
	 * wobei x = yaw, y = pitch, z = roll
	 */
	private Vector3f yprRotation;
	
	/**
	 * Groesse eines Elements des Levels,
	 * wichtig um Weltkoordinaten des Spielers zu berechnen
	 */
	private int tileSize;
	
	/**
	 * Vektoren, die definieren, an welchen Koordinaten sich der Spieler
	 * bewegt, wenn er sich vor/zurueck bewegt, oder seitlich strafed
	 */
	private Vector3f moveFrontBack = new Vector3f(0.0f, 0.0f, 1.0f); 
	private Vector3f moveRightLeft = new Vector3f(1.0f, 0.0f, 0.0f);
	
	/**
	 * Gibt an, ob der Prozess gerade eine Bewegung interpoliert
	 */
	private boolean moving = false;
	
	/**
	 * Das Ziel der Positions-Interpolation
	 */
	private Vector3f goalPosition = new Vector3f();
	
	/**
	 * Die Differenz zwischen Ziel- und aktueller Position
	 */
	private Vector3f diffPosGoalVector;
	
	/**
	 * Bewegungsgeschwindigkeit
	 */
	private float moveSpeed = 0.005f;
	
	/**
     * Setzt den Spieleractor, dessen Position veraendert werden soll.
     * @param actor Der Actor
     * @return <tt>true</tt>, falls der Actor exsitiert und eine
     *         <tt>TransformComponent</tt> besitzt; <tt>false</tt> sonst
     */
    public boolean setPlayer(Actor actor) {
        this.playerPose = null;
        if(actor != null) {
            TransformComponent comp = (TransformComponent)actor.getComponent(TransformComponent.ID);
            if(comp != null) {
                this.playerPose = comp.getPose();
                this.position = new Vector3f();
                this.yprRotation = new Vector3f();
                return true;
            } else {
                Logger.INSTANCE.warning("Actor has no TransformComponent.");
                return false;
            }
        } else {
            Logger.INSTANCE.warning("Actor does not exist");
            return false;
        }
    }
    
    /**
     * Aktualisiert die Pose (Position und Rotation) des Spielers
     */
    private void updatePose() {
    	if(this.playerPose != null) {
    		this.playerPose.setRotation(this.yprRotation.x % MathUtil.PI_MUL2, MathUtil.clamp(this.yprRotation.y, -MathUtil.PI_DIV2, MathUtil.PI_DIV2), 0.0f);
    		// Levelkoordinaten werden in Weltkoordinaten umgerechnet
    		this.playerPose.setPosition(this.position.x * tileSize, this.position.y * tileSize, this.position.z * tileSize);
    	}
    }
    
    /**
     * Setzt die Position des Spielers sofort auf einen bestimmten Wert (in LevelKoordinaten)
     * @param x x-Koordinate
     * @param y y-Koordinate
     * @param z z-Koordinate
     */
    public void setPlayerPosition(float x, float y, float z) {
    	this.position.x = x;
    	this.position.y = y;
    	this.position.z = z;
    	this.updatePose();
    }
    
    /**
     * Startet die Interpolation der Spielerposition zu einer gewuenschten Position (in Levelkoordinaten)
     * @param x x-Koordinate
     * @param y y-Koordinate 
     * @param z z-Koordinate
     */
    public void movePlayerToPosition(float x, float y, float z) {
    	if (this.moving == false) {
    		// Zielposition wird gesetzt
	    	this.goalPosition.x = x;
	    	this.goalPosition.y = y;
	    	this.goalPosition.z = z;
	    	// Differenz wird berechnet und normalisiert
	    	this.diffPosGoalVector = Vector3f.sub(this.goalPosition, this.position, null);
	    	this.diffPosGoalVector.normalise();
	    	this.moving = true;
    	}
    }
    
    /**
     * Berechnet die naechste Spielerposition (in Levelkoordinaten)
     * @param direction Bewegungsrichtung
     * @return Position nach der Bewegung
     */
    public Vector3f calculateNextPlayerPosition(int direction) {
    	Vector3f nextPosition = new Vector3f(this.position);
    	switch (direction) {
	    	case DIR_FORWARD:
	    		nextPosition.x += this.moveFrontBack.x;
	    		nextPosition.z += this.moveFrontBack.z;
	    		break;
	    	case DIR_STRAFELEFT:
	    		nextPosition.x -= this.moveRightLeft.x;
	    		nextPosition.z -= this.moveRightLeft.z;
	    		break;
	    	case DIR_STRAFERIGHT:
	    		nextPosition.x += this.moveRightLeft.x;
	    		nextPosition.z += this.moveRightLeft.z;
	    		break;
	    	case DIR_BACKWARD:
	    		nextPosition.x -= this.moveFrontBack.x;
	    		nextPosition.z -= this.moveFrontBack.z;
	    	default:
	    		break;
    	}
    	return nextPosition;
    }
    
    /**
     * Veraendert die Bewegungsvektoren, damit die Bewegung an die Rotation angepasst ist
     * @param rotDirection Richtung der Rotation
     */
    public void updateMoveVectorsWithRotationDirection(int rotDirection) {
    	switch (rotDirection) {
	    	case ROTATE_LEFT:
	    		Vector3f tmp = new Vector3f(this.moveFrontBack);
	    		this.moveFrontBack.x = -this.moveRightLeft.x;
	    		this.moveFrontBack.z = -this.moveRightLeft.z;
	    		this.moveRightLeft = tmp;
	    		break;
	    	case ROTATE_RIGHT:
	    		tmp = new Vector3f(this.moveFrontBack);
	    		this.moveFrontBack = new Vector3f(this.moveRightLeft);
	    		this.moveRightLeft.x = -tmp.x;
	    		this.moveRightLeft.z = -tmp.z;
	    		break;
    	}
    }
    
    /**
     * Setzt die Groesse eines Level-Elements
     * @param tileSize
     */
    public void setTileSize(int tileSize) {
    	this.tileSize = tileSize;
    }

	@Override
	/**
	 * Diese Funktion wird im ProzessManager bei jedem Frame aufgerufen und es wird die vergangene
	 * Zeit seit dem letzten Frame uebergeben
	 */
	protected void onUpdate(long dMillis) {
		if (this.moving) {
			// Es wird gecheckt, ob wir uns am Ende der Positionsinterpolation befinden
			if ((this.diffPosGoalVector.x == 0.0 /*Es wird in x-Richtung interpoliert*/ || ((this.diffPosGoalVector.x > 0 /*Die Interpolationsrichtung ist positiv*/ && this.position.x >= this.goalPosition.x /*Die aktuelle x-Koordinate uebersteigt die Zielkoordinate*/) || (this.diffPosGoalVector.x < 0 /*Es wird in negative Richtung interpoliert*/ && this.position.x <= this.goalPosition.x /*Die aktuelle x-Koordinate ist kleiner als die Zielkoordinate*/)))
			 && (this.diffPosGoalVector.y == 0.0 || ((this.diffPosGoalVector.y > 0 && this.position.y >= this.goalPosition.y) || (this.diffPosGoalVector.y < 0 && this.position.y <= this.goalPosition.y)))
			 && (this.diffPosGoalVector.z == 0.0 || ((this.diffPosGoalVector.z > 0 && this.position.z >= this.goalPosition.z) || (this.diffPosGoalVector.z < 0 && this.position.z <= this.goalPosition.z)))) {
				// Wenn wir das Ziel erreicht haben,
				// wird die aktuelle Position auf die Zielposition gesetzt (new hier wichtig, um das Objekt zu kopieren)
				this.position = new Vector3f(this.goalPosition);
				this.moving = false;
			} else {
				// Wenn es noch nicht erreicht wurde, wird ein kleiner Wert auf alle zu interpolierenden Koordinaten addiert
				// Und sich dadurch Richtung Ziel bewegt
				float speed = dMillis * moveSpeed;
				this.position.x += this.diffPosGoalVector.x * speed;
				this.position.y += this.diffPosGoalVector.y * speed;
				this.position.z += this.diffPosGoalVector.z * speed;
			}
			// Die Pose des Actors wird aktualisiert
			this.updatePose();
		}
	}
}
