/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package logging;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.logging.Level;

/**
 * Der Logger dient dazu, Ausgaben der Engine moeglichst strukturiert und
 * uebersichtlich zu gestalten.
 * 
 * @author nico3000, Erik Wittkorn
 */
public class Logger {
    /**
     * Einzige Instanz des Loggers.
     */
    public static final Logger INSTANCE = new Logger();
    
    /**
     * Ausgabestream zur Logdatei.
     */
    private BufferedWriter logFileWriter = null;
    
    /**
     * Logging-Infos von Dateinamen, die sich in diesem Filter befinden werden nicht ausgegeben
     */
    ArrayList<String> filter = new ArrayList<String>();

    /**
     * Privater Konstruktor entsprechend dem Singletonprinzip.
     */
    private Logger() {
    }
    
    /**
     * Fuegt dem Filter einen weiteren Dateinamen hinzu
     * @param filename Dateiname, von dem Logging-Ausgaben gefiltert werden sollen
     */
    public void addFileToFilter(String filename) {
    	this.filter.add(filename);
    }
    
    /**
     * Entfernt einen Dateinamen aus dem Filter
     * @param filename Dateiname, von dem Logging-Ausgaben nicht weiter gefiltert werden sollen
     */
    public void removeFileFromFilter(String filename) {
    	int foundIndex = filterContainsIndex(filename);
    	if (foundIndex != -1) {
    		this.filter.remove(foundIndex);
    	}
    }
    
    /**
     * Sucht einen Eintrag im Filter, der im Suchstring vorhanden ist
     * @param value Suchstring mit potentiellem Filter-Eintrag
     * @return Index des gefundenen FilterEintrags, -1 wenn nicht gefunden
     */
    private int filterContainsIndex(String value) {
    	int foundIndex = -1;
    	for (int i = 0; i < this.filter.size(); i++) {
    		String filterValue = this.filter.get(i);
    		if (value.contains(filterValue)) {
    			foundIndex = i;
    		}
    	}
    	return foundIndex;
    }
    
    /**
     * Oeffnet den Stream zur Logdatei.
     * @param filePath Der Pfad zur Datei
     * @return <tt>true</tt>, falls erfolgreich
     */
    public boolean open(String filePath) {
        if(this.logFileWriter != null) {
            try {
                this.logFileWriter.close();
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(Logger.class.getName()).log(Level.SEVERE, null, ex);
                return false;
            }
        }
        try {
            this.logFileWriter = new BufferedWriter(new FileWriter(filePath));
            this.logFileWriter.write("Nicotopia Engine v1.0 - Logfile\n");
        } catch (IOException ex) {
            java.util.logging.Logger.getLogger(Logger.class.getName()).log(Level.SEVERE, null, ex);
            return false;
        }
        return true;
    }
    
    /**
     * Schliesst den Stream zur Logdatei.
     */
    public void close() {
        if(this.logFileWriter != null) {
            try {
                this.logFileWriter.close();
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(Logger.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
    
    /**
     * Druckt eine Zeile in die Datei und auf die Konsole.
     * @param line Die Zeile, die gedruckt werden soll
     * @param asError Falls <tt>true</tt>, wird die Konsolenausgabe in den
     *                Errorstream geschrieben
     */
    private synchronized void print(String line, boolean asError) {
    	
        (asError ? System.err : System.out).print(line);
        if(this.logFileWriter != null) {
            try {
                this.logFileWriter.write(line);
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(Logger.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else {
            System.err.println("[ERROR] Logger not yet opened. Call open() on newly created Logger instance.");
        }        
    }
    
    /**
     * Druckt eine Information der Form <tt>[INFO] Dateiname, Zeile: text</tt>.
     * Der Offset gibt dabei die Anzahl an Dateien an, die zurueckgegangen
     * werden soll, um die Quelle des Textes an eine andere Stelle zu setzen.
     * @param text Der zu erscheinende Text
     * @param backtrackOffset Der Offset des Backtracking
     */
    public void info(String text) {
    	String srcString = this.getSrcString();
    	if (filterContainsIndex(srcString) != -1) {
    		return;
    	}
        String line = String.format("[INFO] %s%s\n", this.getSrcString(), text);
        this.print(line, false);
    }
    
    /**
     * Druckt eine Warnung der Form <tt>[WARNING] Dateiname, Zeile: text</tt>.
     * Der Offset gibt dabei die Anzahl an Dateien an, die zurueckgegangen
     * werden soll, um die Quelle des Textes an eine andere Stelle zu setzen.
     * @param text Der zu erscheinende Text
     * @param backtrackOffset Der Offset des Backtracking
     */
    public void warning(String text) {
    	String srcString = this.getSrcString();
    	if (filterContainsIndex(srcString) != -1) {
    		return;
    	}
        String line = String.format("[WARNING] %s%s\n", this.getSrcString(), text);
        this.print(line, false);
    }
    
    /**
     * Druckt einen Fehler der Form <tt>[ERROR] Dateiname, Zeile: text</tt>. Der
     * Offset gibt dabei die Anzahl an Dateien an, die zurueckgegangen werden
     * soll, um die Quelle des Textes an eine andere Stelle zu setzen.
     * @param text Der zu erscheinende Text
     * @param backtrackOffset Der Offset des Backtracking
     */
    public void error(String text) {
    	String srcString = this.getSrcString();
    	if (filterContainsIndex(srcString) != -1) {
    		return;
    	}
        String line = String.format("[ERROR] %s%s\n", this.getSrcString(), text);
        this.print(line, true);
    }
    
    /**
     * Liefert den String, der Dateiname und Zeile enthaelt.
     * @param backtrackOffset Der Offset des Backtracking
     * @return Ein String der Form <tt>Dateiname, Zeile:</tt>
     */
    private String getSrcString() {
        StackTraceElement stack[] = Thread.currentThread().getStackTrace();
        String file = stack[stack.length - 1].getFileName();
        int line = stack[stack.length - 1].getLineNumber();
        return String.format("%s, %d: ", file, line);
    }

}
