package parser;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import model.Actor;
import model.BoundingSphere;
import model.Box;
import model.components.ActorComponent;
import model.components.PhysicsComponent;
import model.components.RenderComponent;
import model.components.TransformComponent;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * Prototypische Klasse, in der eine <Scene>-XML Datei nach
 * dem DOM-Modell geparst wird
 * 
 * @author Erik Wittkorn
 */
public class DOMSceneParser {
	
/**
 * Bekommt als Eingabe den Namen einer XML-Datei, die nach dem DOM-Model geparst wird
 * und gibt eine Liste mit Instanzen der model-Klassen zuerueck
 * @param filename Name der XML-Datei
 * @return Liste mit gefuellten Instanzen der model-Klassen
 */
	public static ArrayList<Actor> parse(String filename) {
		ArrayList<Actor> results = new ArrayList<Actor>();
		
		try {
			
			// Die Datei wird durch ihren Namen referenziert
			File xmlFile = new File(filename);
			
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder;
			dBuilder = dbFactory.newDocumentBuilder();
			
			// Die XML-Datei wird zu einem Document-Objekt geparst
			Document xmlDoc = dBuilder.parse(xmlFile);
			
			// Die Root-Node des Documents ist der <scene>-tag
			Node sceneNode = xmlDoc.getDocumentElement();
			
			// Das erste Kind der Scene-Node duerfte ein Actor sein
			Node actorNode = sceneNode.getFirstChild();
			
			while (actorNode != null) {
				switch (actorNode.getNodeName()) {
					case "Actor":
						
						// Das Attribut Name wird als String gelesen und damit ein neuer Actor erstellt
						String name = actorNode.getAttributes().getNamedItem("name").getNodeValue();
						Actor currentActor = new Actor(name);
						
						// Als Kindknoten hat ein Actor nur Komponenten, die speziell erstellt, und dem Actor hinzugefuegt werden
						Node componentNode = actorNode.getFirstChild();
						while (componentNode != null) {
							// In dieser Hilfsfunktion wird eine Actor-Komponente aus einer XML-Node generiert
							ActorComponent component = DOMSceneParser.createActorComponent(componentNode);
							if (component != null) {
								currentActor.addComponent(component);
							}
							// Die naechste Komponente wird aufgerufen
							componentNode = componentNode.getNextSibling();
						}
						
						// Der Actor mit den Komponenten wird der Liste hinzugefuegt
						results.add(currentActor);
						break;
						
					case "#text":
						// Nothing to do here
						break;
						
					default:
						// Nothing to do here
						break;
						
				}
				// Die naechste Actor-Node wird zur Bearbeitung eingelesen
				actorNode = actorNode.getNextSibling();
			}
		} catch (ParserConfigurationException | SAXException | IOException e) {
			e.printStackTrace();
		}
		
		
		return results;
	}
	
	/**
	 * Hilfsfunktion, die eine Komponenten-Node parst und daraus eine
	 * Actorkomponente mit entsprechenenden Eigenschaften erstellt
	 * @param componentNode die XML-Node einer Actorkomponente
	 * @return die aus der XML generierte Actorkomponente
	 */
	private static ActorComponent createActorComponent(Node componentNode) {
		ActorComponent componentResult = null;
		
		switch (componentNode.getNodeName()) {
		
			case "TransformComponent":
				TransformComponent transComp = new TransformComponent();
				Node transCompChildNode = componentNode.getFirstChild();
				
				while (transCompChildNode != null) {
					switch (transCompChildNode.getNodeName()) {
					
						case "Position":
							float x = Float.parseFloat(transCompChildNode.getAttributes().getNamedItem("x").getTextContent());
							float y = Float.parseFloat(transCompChildNode.getAttributes().getNamedItem("y").getTextContent());
							float z = Float.parseFloat(transCompChildNode.getAttributes().getNamedItem("z").getTextContent());
							transComp.setPosition(x, y, z);
							break;
							
						case "Rotation":
							float yaw = Float.parseFloat(transCompChildNode.getAttributes().getNamedItem("yaw").getTextContent());
							float pitch = Float.parseFloat(transCompChildNode.getAttributes().getNamedItem("pitch").getTextContent());
							float roll = Float.parseFloat(transCompChildNode.getAttributes().getNamedItem("roll").getTextContent());
							transComp.setRotation(yaw, pitch, roll);
							break;
							
						default:
							//Nothing to do here
							break;
							
					}
					
					transCompChildNode = transCompChildNode.getNextSibling();
				}
				
				componentResult = transComp;
				break;
				
			case "PhysicsComponent":
				PhysicsComponent physicsComp = new PhysicsComponent();
				Node physicsCompChildNode = componentNode.getFirstChild();
				
				while (physicsCompChildNode != null) {
					switch (physicsCompChildNode.getNodeName()) {
						
						case "Box":
							Box box = new Box();
							Node boxChildNode = physicsCompChildNode.getFirstChild();
							
							while (boxChildNode != null) {
								switch (boxChildNode.getNodeName()) {
									case "Size":
										float x = Float.parseFloat(boxChildNode.getAttributes().getNamedItem("x").getTextContent());
										float y = Float.parseFloat(boxChildNode.getAttributes().getNamedItem("y").getTextContent());
										float z = Float.parseFloat(boxChildNode.getAttributes().getNamedItem("z").getTextContent());
										box.setSize(x, y, z);
										break;
										
									case "Density":
										float density = Float.parseFloat(boxChildNode.getAttributes().getNamedItem("value").getTextContent());
										box.setDensity(density);
										break;
										
									default:
										// Nothing to do here
										break;
								}
								
								boxChildNode = boxChildNode.getNextSibling();
							}
							
							physicsComp.addBox(box);
							break;
						
						default:
							//Nothing to do here...
							break;
					}
					
					physicsCompChildNode = physicsCompChildNode.getNextSibling();
				}
				
				componentResult = physicsComp;
				break;

			case "RenderComponent":
				RenderComponent renderComp = new RenderComponent();
				Node renderCompChildNode = componentNode.getFirstChild();
				
				while (renderCompChildNode != null) {
					switch (renderCompChildNode.getNodeName()) {
						case "ObjMesh":
							String value = renderCompChildNode.getAttributes().getNamedItem("resource").getTextContent();
							renderComp.setObjMesh(value);
							break;

						case "BoundingSphere":
							BoundingSphere bSphere = new BoundingSphere();
							float radius = Float.parseFloat(renderCompChildNode.getAttributes().getNamedItem("radius").getTextContent());
							bSphere.setRadius(radius);
							renderComp.addBoundingSphere(bSphere);
							break;
							
						default:
							break;
					}
					
					renderCompChildNode = renderCompChildNode.getNextSibling();
				}
				
				componentResult = renderComp;
				break;
			
				
			default:
				
				break;
		}
		
		return componentResult;
	}

}
