package parser.sax;

import java.util.ArrayList;

import model.Actor;
import model.BoundingSphere;
import model.Box;
import model.components.PhysicsComponent;
import model.components.RenderComponent;
import model.components.TransformComponent;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Handler-Klasse, die die Events verarbeitet, die beim SAX-Parsing
 * einer <Scene>-XML-Datei auftreten
 * 
 * @author Master Erik Wittkorn
 */
public class SAXSceneParserHandler implements ContentHandler {
	
	private final ArrayList<Actor> actorList;
	
	/**
	 * Platzhalter-Referenzen fuer die Objekte, die durch die
	 * XML erstellt werden
	 */
	private Actor currentActor = null;
	
	private RenderComponent currentRenderComp = null;
	private PhysicsComponent currentPhysicsComp = null;
	private TransformComponent currentTransformComp = null;
	
	private Box currentBox = null;
	
	/**
	 * Im Konstruktor wird die Liste fuer die Ergebnisse des
	 * Parsing uebergeben
	 * @param actorList Leere Actor-Liste
	 */
	public SAXSceneParserHandler (ArrayList<Actor> actorList) {
		this.actorList = actorList;
	}

	@Override
	public void startDocument() throws SAXException {
		
	}
	
	/**
	 * Wird aufgerufen, wenn ein Start-Tag beim Parsing vorkommt
	 */
	@Override
	public void startElement(String uri, String localName, String qName,
			Attributes atts) throws SAXException {
		switch (localName) {
		
			// Objekte mit Kindknoten werden erstellt und ihre Referenz in der Hilsvariablen abgelegt
			case "Actor":
				String name = atts.getValue("name");
				currentActor = new Actor(name);
				break;
				
			case "TransformComponent":
				TransformComponent transComp = new TransformComponent();
				if (currentTransformComp == null) {
					currentTransformComp = transComp;
				} else {
					System.out.println("Something awful TransformComp");
				}
				break;
				
			case "PhysicsComponent":
				PhysicsComponent physicsComp = new PhysicsComponent();
				if (currentPhysicsComp == null) {
					currentPhysicsComp = physicsComp;
				} else {
					System.out.println("Something awful PhysicsComp");
				}
				break;
				
			case "RenderComponent":
				RenderComponent renderComp = new RenderComponent();
				if (currentRenderComp == null) {
					currentRenderComp = renderComp;
				} else {
					System.out.println("Something awful RenderComp");
				}
				break;
				
			case "Box":
				Box box = new Box();
				if (currentBox == null) {
					currentBox = box;
				} else {
					System.out.println("Something awful Box");
				}
				break;
			
			// Elemente, die nur Attribute haben, werden bei dem Objekt ihres Vater-Elements
			// sofort eingestellt
			case "Position":
				float x = Float.parseFloat(atts.getValue("x"));
				float y = Float.parseFloat(atts.getValue("y"));
				float z = Float.parseFloat(atts.getValue("z"));
				if (currentTransformComp != null) {
					currentTransformComp.setPosition(x, y, z);
				}
				break;
				
			case "Rotation":
				float yaw = Float.parseFloat(atts.getValue("yaw"));
				float pitch = Float.parseFloat(atts.getValue("pitch"));
				float roll = Float.parseFloat(atts.getValue("roll"));
				if (currentTransformComp != null) {
					currentTransformComp.setRotation(yaw, pitch, roll);
				}
				break;
				
			case "Size":
				x = Float.parseFloat(atts.getValue("x"));
				y = Float.parseFloat(atts.getValue("y"));
				z = Float.parseFloat(atts.getValue("z"));
				if (currentBox != null) {
					currentBox.setSize(x, y, z);
				}
				break;
			
			case "Density":
				float density = Float.parseFloat(atts.getValue("value"));
				if (currentBox != null) {
					currentBox.setDensity(density);
				}
				break;
			
			case "ObjMesh":
				String resource = atts.getValue("resource");
				if (currentRenderComp != null) {
					currentRenderComp.setObjMesh(resource);
				} else {
					System.out.println("Something awful Render + Mesh");
				}
				break;
			
			case "BoundingSphere":
				float radius = Float.parseFloat(atts.getValue("radius"));
				BoundingSphere bSphere = new BoundingSphere();
				bSphere.setRadius(radius);
				if (currentRenderComp != null) {
					currentRenderComp.addBoundingSphere(bSphere);
				} else {
					System.out.println("Something awful Render + bSphere");
				}
				break;
		}
		
	}
	
	/**
	 * Diese Methode wird aufgerufen, wenn ein End-Tag beim XML-Parsing
	 * vorgekommen ist
	 */
	@Override
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		switch (localName) {
		
			/**
			 * Der Actor wird der Liste hinzugefuegt
			 */
			case "Actor":
				actorList.add(currentActor);
				currentActor = null;
				break;
			
			/**
			 * Die Komponenten werden dem Actor hinzugefuegt
			 */
			case "TransformComponent":
				if (currentActor != null) {
					currentActor.addComponent(currentTransformComp);
				} else {
					System.out.println("Actor ist null :( Transform");
				}
				currentTransformComp = null;
				break;
				
			case "PhysicsComponent":
				if (currentActor != null) {
					currentActor.addComponent(currentPhysicsComp);
				} else {
					System.out.println("Actor ist null :( Physic");
				}
				currentRenderComp = null;
				break;
				
			case "RenderComponent":
				if (currentActor != null) {
					currentActor.addComponent(currentRenderComp);
				} else {
					System.out.println("Actor ist null :( Render");
				}
				currentRenderComp = null;
				break;
			
			// Die Box wird der RenderKomponente hinzugefuegt
			case "Box":
				if (currentPhysicsComp != null) {
					currentPhysicsComp.addBox(currentBox);
				}
				currentBox = null;
				break;
			
			default:
				// Nothing to do here...
				break;
		}
		
	}
	
	@Override
	public void endDocument() throws SAXException {
		
	}
	
	
	
	
	
	
	
	@Override
	public void characters(char[] ch, int start, int length)
			throws SAXException {
	}

	@Override
	public void endPrefixMapping(String prefix) throws SAXException {
	}

	@Override
	public void ignorableWhitespace(char[] ch, int start, int length)
			throws SAXException {
	}

	@Override
	public void processingInstruction(String target, String data)
			throws SAXException {
	}

	@Override
	public void setDocumentLocator(Locator locator) {
	}

	@Override
	public void skippedEntity(String name) throws SAXException {
	}


	@Override
	public void startPrefixMapping(String prefix, String uri)
			throws SAXException {
	}

}
