package writer;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import model.Actor;
import model.BoundingSphere;
import model.Box;
import model.components.PhysicsComponent;
import model.components.RenderComponent;
import model.components.TransformComponent;

import org.lwjgl.util.vector.Vector3f;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Hilfsklasse, um aus einer Actor-Liste eine entsprechende XML-Datei
 * zu generieren
 * 
 * @author Erik Wittkorn
 */
public class SceneXMLWriter {

	/**
	 * Durch Hilfsfunktionen wird hier aus einer Actor-Liste eine
	 * entsprechende XML-Datei generiert und in eine gewuenschte
	 * Datei geschrieben
	 * @param actorList Liste mit Actors
	 * @param fileName gequenschter Dateiname fuer die Ausgabe-XML
	 */
	public static void writeSceneXML(ArrayList<Actor> actorList, String fileName) {
		Document sceneDocument = createSceneDocument(actorList);
		try {
			FileOutputStream fileStream = new FileOutputStream(new File(fileName));
			printDocument(sceneDocument, fileStream);
			fileStream.close();
		} catch (IOException | TransformerException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * In dieser Hilfsfunktion wird aus einer Actor-Liste ein
	 * Document-Object generiert, das den Inhalt der Liste repraesentiert
	 * @param actorList Liste mit Actors
	 * @return Document-Object der entsprechenden XML
	 */
	private static Document createSceneDocument(ArrayList<Actor> actorList) {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		Document sceneDocument = null;
		DocumentBuilder builder;
		try {
			builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();
			
			Element sceneElement = document.createElement("Scene");
			document.appendChild(sceneElement);
			
			for (Actor actor : actorList) {
				Element actorElement = document.createElement("Actor");
				actorElement.setAttribute("name", actor.getName());
				
				TransformComponent transComp = (TransformComponent)actor.getComponent("TransformComponent");
				if (transComp != null) {
					Element transCompElement = document.createElement("TransformComponent");
					
					Vector3f position = transComp.getPosition();
					Element positionElement = document.createElement("Position");
					positionElement.setAttribute("x", String.valueOf(position.x));
					positionElement.setAttribute("y", String.valueOf(position.y));
					positionElement.setAttribute("z", String.valueOf(position.z));
					transCompElement.appendChild(positionElement);
					
					Vector3f rotation = transComp.getRotation();
					Element rotationElement = document.createElement("Rotation");
					rotationElement.setAttribute("yaw", String.valueOf(rotation.x));
					rotationElement.setAttribute("pitch", String.valueOf(rotation.y));
					rotationElement.setAttribute("roll", String.valueOf(rotation.z));
					transCompElement.appendChild(rotationElement);
					
					actorElement.appendChild(transCompElement);
				}
				
				PhysicsComponent physicsComp = (PhysicsComponent)actor.getComponent("PhysicsComponent");
				if (physicsComp != null) {
					Element physicsCompElement = document.createElement("PhysicsComponent");
					
					ArrayList<Box> boxes = physicsComp.getBoxes();
					for (Box box : boxes) {
						Element boxElement = document.createElement("Box");
						
						Vector3f size = box.getSize();
						Element sizeElement = document.createElement("Size");
						sizeElement.setAttribute("x", String.valueOf(size.x));
						sizeElement.setAttribute("y", String.valueOf(size.y));
						sizeElement.setAttribute("z", String.valueOf(size.z));
						boxElement.appendChild(sizeElement);
						
						float density = box.getDensity();
						Element densityElement = document.createElement("Density");
						densityElement.setAttribute("value", String.valueOf(density));
						boxElement.appendChild(densityElement);
						
						physicsCompElement.appendChild(boxElement);
					}
					
					actorElement.appendChild(physicsCompElement);
				}

				RenderComponent renderComp = (RenderComponent)actor.getComponent("RenderComponent");
				if (renderComp != null) {
					Element renderCompElement = document.createElement("RenderComponent");
					
					String objMesh = renderComp.getObjMesh();
					Element objMeshElement = document.createElement("ObjMesh");
					objMeshElement.setAttribute("resource", objMesh);
					renderCompElement.appendChild(objMeshElement);
					
					ArrayList<BoundingSphere> spheres = renderComp.getBoundingSpheres();
					for (BoundingSphere bSphere : spheres) {
						Element bSphereElement = document.createElement("BoundingSphere");
						
						float radius = bSphere.getRadius();
						bSphereElement.setAttribute("radius", String.valueOf(radius));
						
						renderCompElement.appendChild(bSphereElement);
					}
					actorElement.appendChild(renderCompElement);
				}
				
				sceneElement.appendChild(actorElement);
				sceneDocument = document;
			}
			
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return sceneDocument;
	}
	
	/**
	 * Hilfsfunktion, um ein beliebiges Document-Object als XML formatiert in einen
	 * beliebigen OutputStream zu schreiben
	 * @param doc Document-Object der XML
	 * @param out OutputStream fuer das XML-Ergebnis
	 * @throws IOException
	 * @throws TransformerException
	 */
	private static void printDocument(Document doc, OutputStream out) throws IOException, TransformerException {
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer transformer = tf.newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
        transformer.setOutputProperty(OutputKeys.METHOD, "xml");
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
        transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");

        transformer.transform(new DOMSource(doc), 
             new StreamResult(new OutputStreamWriter(out, "UTF-8")));
    }
}
